import sys

from hakisto import logger


def color():
    logger.file_handler.severity = sys.maxsize
    logger.set_severity(0)

    logger.trace('Trace')
    logger.debug('Debug')
    logger.verbose('Verbose')
    logger.info('Info')
    logger.success('Success')
    logger.warning('Warning')
    logger.error('Error')


def critical():
    s = 'Hakisto'
    msg = f'Colors with {s} is easy!'
    logger.critical('A critical issue!')


def traceback():
    s = 'Hakisto'
    msg = f'Colors with {s} is easy!'
    b = s[10]
